'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _CSSCore = require('fbjs/lib/CSSCore');

var _CSSCore2 = _interopRequireDefault(_CSSCore);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _ProgressBar = require('./ProgressBar');

var _ProgressBar2 = _interopRequireDefault(_ProgressBar);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); } //  eslint-disable-line no-unused-vars


var ToastrBox = function (_Component) {
  _inherits(ToastrBox, _Component);

  function ToastrBox(props) {
    _classCallCheck(this, ToastrBox);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this._onAnimationComplete = function () {
      var _this$props = _this.props;
      var remove = _this$props.remove;
      var item = _this$props.item;
      var options = item.options;
      var id = item.id;


      if (_this.isHiding) {
        _this._setIsHiding(false);
        remove(id);
        if (options.onHideComplete) {
          options.onHideComplete();
        }
      } else if (!_this.isHiding && options.onShowComplete) {
        options.onShowComplete();
      }
    };

    _this._removeToastr = function () {
      if (!_this.isHiding) {
        _this._setIsHiding(true);
        _this._setTransition(true);
        (0, _utils.onCSSTransitionEnd)(_this.toastrBox, _this._onAnimationComplete);
      }
    };

    _this._setTransition = function (hide) {
      var node = _this.toastrBox;
      var animationType = hide ? _this.transitionOut : _this.transitionIn;

      var onEndListener = function onEndListener(e) {
        if (e && e.target == node) {
          _CSSCore2.default.removeClass(node, animationType);
        }
      };

      (0, _utils.onCSSTransitionEnd)(_this.toastrBox, onEndListener);
      _CSSCore2.default.addClass(node, animationType);
    };

    _this._clearTransition = function () {
      var node = _this.toastrBox;
      _CSSCore2.default.removeClass(node, _this.transitionIn);
      _CSSCore2.default.removeClass(node, _this.transitionOut);
    };

    _this._setIntervalId = function (intervalId) {
      _this.intervalId = intervalId;
    };

    _this._setIsHiding = function (val) {
      _this.isHiding = val;
    };

    _this._renderSubComponent = function (SubComponent) {
      var removeCurrentToastrFunc = function removeCurrentToastrFunc() {
        return _this.props.remove(_this.props.item.id);
      };

      if ((0, _react.isValidElement)(SubComponent)) {
        return _react2.default.cloneElement(SubComponent, {
          remove: removeCurrentToastrFunc
        });
      }

      return _react2.default.createElement(SubComponent, { remove: removeCurrentToastrFunc });
    };

    var _props$item = props.item;
    var transitionIn = _props$item.transitionIn;
    var transitionOut = _props$item.transitionOut;


    _this.isHiding = false;
    _this.intervalId = null;

    _this.transitionIn = transitionIn || _this.props.transitionIn;
    _this.transitionOut = transitionOut || _this.props.transitionOut;

    _this.state = { progressBar: null };
    return _this;
  }

  ToastrBox.prototype.componentDidMount = function componentDidMount() {
    var item = this.props.item;

    if (this.props.inMemory[item.id]) return;

    var timeOut = this._getItemTimeOut();

    if (timeOut) {
      this._setIntervalId(setTimeout(this._removeToastr, timeOut));
    }

    if (timeOut && item.options.progressBar) {
      this.setState({ progressBar: { duration: this._getItemTimeOut() } });
    }

    this._setTransition();
    (0, _utils.onCSSTransitionEnd)(this.toastrBox, this._onAnimationComplete);
    this.props.addToMemory(item.id);
  };

  ToastrBox.prototype.componentWillUnmount = function componentWillUnmount() {
    if (this.intervalId) {
      clearTimeout(this.intervalId);
    }
  };

  ToastrBox.prototype.handleClick = function handleClick() {
    this._removeToastr();
  };

  ToastrBox.prototype.mouseEnter = function mouseEnter() {
    clearTimeout(this.intervalId);
    this._setIntervalId(null);
    this._setIsHiding(false);

    var progressBar = this.props.item.options.progressBar;

    var timeOut = this._getItemTimeOut();
    if (timeOut && progressBar) {
      this.setState({ progressBar: null });
    }
  };

  ToastrBox.prototype.mouseLeave = function mouseLeave() {
    var removeOnHover = this.props.item.options.removeOnHover;

    if (!this.isHiding && this.props.item.type !== 'message' && removeOnHover == true) {
      this._setIntervalId(setTimeout(this._removeToastr, 1000));
      var progressBar = this.props.item.options.progressBar;

      var timeOut = this._getItemTimeOut();
      if (timeOut && progressBar) {
        this.setState({ progressBar: { duration: 1000 } });
      }
    }
  };

  ToastrBox.prototype._getItemTimeOut = function _getItemTimeOut() {
    var item = this.props.item;
    var timeOut = item.options.timeOut;

    if (typeof timeOut === 'undefined' && item.type !== 'message') {
      timeOut = this.props.timeOut;
    }

    return timeOut;
  };

  ToastrBox.prototype.render = function render() {
    var _this2 = this;

    var _props$item2 = this.props.item;
    var options = _props$item2.options;
    var type = _props$item2.type;
    var message = _props$item2.message;
    var title = _props$item2.title;

    return _react2.default.createElement(
      'div',
      {
        ref: function ref(_ref) {
          return _this2.toastrBox = _ref;
        },
        className: (0, _classnames2.default)('toastr', 'animated', type, options.icon, options.className),
        onMouseEnter: this.mouseEnter.bind(this),
        onMouseLeave: this.mouseLeave.bind(this)
      },
      _react2.default.createElement(
        'div',
        { className: 'message-holder' },
        title && _react2.default.createElement(
          'div',
          { className: 'title' },
          title
        ),
        message && _react2.default.createElement(
          'div',
          { className: 'message' },
          message
        ),
        options.component && _react2.default.createElement(
          'div',
          { className: 'message' },
          this._renderSubComponent(options.component)
        )
      ),
      options.showCloseButton && _react2.default.createElement(
        'button',
        {
          type: 'button',
          className: 'close-toastr',
          onClick: this.handleClick.bind(this)
        },
        'x'
      ),
      this.state.progressBar ? _react2.default.createElement(_ProgressBar2.default, this.state.progressBar) : null
    );
  };

  return ToastrBox;
}(_react.Component);

ToastrBox.displayName = 'ToastrBox';
ToastrBox.propTypes = {
  item: _react.PropTypes.object.isRequired
};
exports.default = ToastrBox;